import calendar
import time
import uuid
from datetime import date, datetime
from django.apps import apps
from django.utils import timezone
import pytz
from django.conf import settings
from django.utils.dateparse import parse_datetime




def local_datetime(s: str, tz: str = settings.TIME_ZONE)-> datetime:
    naive = parse_datetime(s)
    if not naive:
        raise ValueError("local_datetime: Not a valid datetime")

    return pytz.timezone(tz).localize(naive, is_dst=None)

def get_client_ip(request):
    META_PRECEDENCE_ORDER = (
        'HTTP_X_FORWARDED_FOR', 'X_FORWARDED_FOR',  # <client>, <proxy1>, <proxy2>
        'HTTP_CLIENT_IP',
        'HTTP_X_REAL_IP',
        'HTTP_X_FORWARDED',
        'HTTP_X_CLUSTER_CLIENT_IP',
        'HTTP_FORWARDED_FOR',
        'HTTP_FORWARDED',
        'HTTP_VIA',
        'REMOTE_ADDR',
    )

    for h in META_PRECEDENCE_ORDER:
        ip = request.META.get(h, None)
        if ip:
            if ',' in ip:
                ip = ip.split(',')[0]
            return ip
    return None

def generate_int_uuid(size=None):
    u = uuid.uuid1()
    n_random = '{}'.format(u.time_low)

    time_epoch = str(calendar.timegm(time.gmtime()))

    u_id = '{}{}'.format(n_random, time_epoch)

    if size:
        u_id = u_id[:size]
    return int(u_id)

def get_upload_path(instance, filename):
    folder = instance._meta.model_name
    if instance and hasattr(instance, 'customer'):
        path = '{}/'.format(instance.customer.shortname)
    
    path += '{}'.format(folder) if folder.endswith('/') else '{}/'.format(folder)

    today = timezone.now()
    date_path = today.strftime('%Y/%m/%d/')
    path = '{}{}'.format(path, date_path)
    path = '{}{}'.format(path, filename)
    return path

def check_perm(user, action, project=None):
    from nets_core.models import Permission
    
    if user.is_superuser:
        return True
    
    if project:
        try:
            project_member_model = apps.get_model(settings.NETS_CORE_PROJECT_MEMBER_MODEL)
            try:
                member = project_member_model.objects.get(user=user, project=project)
                if hasattr(member, 'enabled') and not member.enabled:
                    return False
                if hasattr(member, 'is_superuser') and member.is_superuser:
                    return True
            except project_member_model.DoesNotExist:
                return False
        except:
            raise Exception('check_perm failed NETS_CORE_PROJECT_MEMBER_MODEL not set in settings')
        
    if not Permission.objects.filter(codename=action, project=project).exists():
        permission = Permission.objects.create(codename=action, project=project)
        return False
    permisions = Permission.objects.filter(
            roles__user=user, 
            roles__project=project,
            roles__enabled=True,
            project=project, 
            codename=action.lower(), 
        ).exists()
    
    if permisions:
        return True
    return False



from django.conf import settings
from nets_core.models import VerificationCode
from oauthlib import common
from oauth2_provider.models import Application, AccessToken, RefreshToken
from django.utils.translation import gettext_lazy as _
from django.utils import timezone

# TODO: create middleware to restring token_access with device_uuid

def validate_verification_code(user, code: str):
    vcode = VerificationCode.objects.filter(user=user).last()
    if not vcode:
        return False
    return vcode.validate(code)


def authenticate(user, code: str, client_id: str, client_secret: str, device_uuid: str=None) -> dict:
    """
        Check client_id and client_secret, validate verification code and
        create access and refresh tokens for oauth2 
        raise Exception if any of this fail.

        Parameters:
        user (instance): Instance of settings.AUTH_MODEL_MODEL

        Returns:
        dict: {"access_token": str, "refresh_token": str, "token_expires": datetime }

        Raise:
        Exception: If client_id, client_secret or code are not valid.

        example of usage:
            try:
                tokens = authenticate(user, '123456', 'client_id_app', 'client_secret_app')
                # return tokens to your user here

            except Exception as e:
                msg = e.__str__()
                # your code if fail here
    """
    try:
        oauth_app = Application.objects.get(client_id=client_id)
        if not oauth_app.client_secret == client_secret:
            raise Exception(_("Invalid client_secret"))
    except Application.DoesNotExist:
        raise Exception(_("Invalid client_id"))
    
    vcode = VerificationCode.objects.filter(user=user).order_by('-created').first()
    if not vcode:
        raise Exception(_("User has not requested verification code"))
    
    if not vcode.validate(code, device_uuid=device_uuid):
        raise Exception(_("Invalid code for this user and device"))
    
    if vcode.device:
        vcode.device.last_login = timezone.now()
        vcode.device.save()
    
    # update code as verified
    vcode.verified = True
    vcode.save()

    if hasattr(user, 'email_verified'):
        user.email_verified = True
    user.last_login = timezone.now()
    user.save()

    # Create access and refresh token 
    expire_seconds = 60*60*24*30 # 30 days as default
    try:
        expire_seconds = settings.ACCESS_TOKEN_EXPIRE_SECONDS
    except:
        pass

    expires = timezone.now() + timezone.timedelta(seconds=expire_seconds)

    access_token = AccessToken.objects.create(
        user=user,
        expires=expires,
        scope='',
        token=common.generate_token(),
        application=oauth_app)


    refresh_token = RefreshToken.objects.create(
        user=user,
        token=common.generate_token(),
        application=oauth_app,
        access_token=access_token
    )
    return {
        "access_token": access_token.token,
        "refresh_token": refresh_token.token,
        "token_expire": access_token.expires
    }



import json
import shortuuid

from uuid import uuid4


from django.conf import settings
from django.apps import apps
from django.db import models
from django.utils.translation import gettext_lazy as _
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import GenericForeignKey
from django.core.cache import cache
from django.contrib.auth.hashers import check_password, make_password
from django.utils import timezone
from django.contrib.auth import get_user_model


from nets_core.utils import generate_int_uuid


User = get_user_model()

token_timeout_seconds = 15*60 # 15 minutes default
try:
    token_timeout_seconds = settings.NETS_CORE_VERIFICATION_CODE_EXPIRE_SECONDS
except Exception as e:
    # Settings not present use default
    pass

class OwnedModel(models.Model):
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    created = models.DateTimeField(_('Created'), auto_now_add=True)
    updated = models.DateTimeField(_('updated'), auto_now=True)

    class Meta:
        abstract = True

class Permission(models.Model):
    name = models.CharField(_("Name"), max_length=150)
    codename = models.CharField(_("Codename"), max_length=150)
    description = models.CharField(_("Description"), max_length=250, null=True, blank=True)
    project_content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE, null=True, blank=True, related_name='nets_core_permissions')
    project_id = models.PositiveIntegerField(null=True, blank=True)
    project = GenericForeignKey('project_content_type', 'project_id')

    class Meta:
        verbose_name = _("Permission")
        verbose_name_plural = _("Permissions")
        db_table = 'nets_core_permission'
        indexes = [
            models.Index(fields=['project_content_type', 'project_id'], name='permission_index')
        ]

    def __str__(self):
        if self.project:
            return f'{self.name} - {self.codename} - {self.project}'
        return f'{self.name} - {self.codename}'
    
    def save(self, *args, **kwargs):
        self.codename = self.codename.lower()
        if not self.name:
            self.name = self.codename
            
        super(Permission, self).save(*args, **kwargs)
        
class Role(models.Model):
    name = models.CharField(_("Name"), max_length=150)
    codename = models.CharField(_("Codename"), max_length=150)
    description = models.CharField(_("Description"), max_length=250)
    permissions = models.ManyToManyField(Permission, related_name='roles')
    project_content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE, null=True, blank=True)
    project_id = models.PositiveIntegerField(null=True, blank=True)
    project = GenericForeignKey('project_content_type', 'project_id')
    enabled = models.BooleanField(_("Enabled?"), default=True)

    class Meta:
        verbose_name = _("Role")
        verbose_name_plural = _("Roles")
        db_table = 'nets_core_role'
        indexes = [
            models.Index(fields=['project_content_type', 'project_id'], name='role_index')
        ]

    def __str__(self):
        if self.project:
            return f'{self.name} - {self.project}'
        return self.name
        
    def save(self, *args, **kwargs):
        self.codename = self.codename.lower()
        super(Role, self).save(*args, **kwargs)
        
class UserRole(models.Model):
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    role = models.ForeignKey(Role, on_delete=models.CASCADE)
    project_content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE, null=True, blank=True)
    project_id = models.PositiveIntegerField(null=True, blank=True)
    project = GenericForeignKey('project_content_type', 'project_id')

    class Meta:
        verbose_name = _("User Role")
        verbose_name_plural = _("User Roles")
        db_table = 'nets_core_user_role'

    def __str__(self):
        if self.project:
            return f'{self.user} - {self.role} - {self.project}'
        return f'{self.user} - {self.role}'    
    


class VerificationCode(OwnedModel):
    token = models.CharField(_("Verification token"), max_length=150)
    device = models.ForeignKey("nets_core.UserDevice", max_length=150, null=True, blank=True, on_delete=models.CASCADE)
    verified = models.BooleanField(_("Verified?"), default=False)
    ip = models.CharField(_("IP"), max_length=150, null=True, blank=True)

    class Meta:
        db_table = 'nets_core_verification_code'
    
    def get_token_cache_key(self):
        token_key_prefix = 'NC_T'
        try:
            token_key_prefix = settings.NETS_CORE_VERIFICATION_CODE_CACHE_KEY 
        except:
            pass

        return f'{token_key_prefix}{self.user.pk}'

    def save(self, *args, **kwargs):
        token = 123456
        cache_token_key = self.get_token_cache_key()
        tester_emails = ['google_tester*']
        # check if settings has testers emails
        if hasattr(settings, 'NETS_CORE_TESTERS_EMAILS') and type(settings.TESTERS_EMAILS) == list:
            tester_emails += settings.NETS_CORE_TESTERS_EMAILS
        
        is_tester = False
        for tester_email in tester_emails:
            # emails can be like google_tester* or google_tester1@gmail.com
            if tester_email.endswith('*'):
                if self.user.email.startswith(tester_email.replace('*', '')):
                    is_tester = True
                    break
            else:
                if self.user.email == tester_email:
                    is_tester = True
                    break

        if is_tester:
            token = 789654
            if hasattr(settings, 'NETS_CORE_TESTERS_VERIFICATION_CODE'):
                token = settings.NETS_CORE_TESTERS_VERIFICATION_CODE

        if not settings.DEBUG and not is_tester:
            # Check cache if token is present and return the same token
            token = cache.get(cache_token_key)
            if not token:
                # Generate a new numeric token six digits
                token = generate_int_uuid(6)
        
        # Set token in cache system 
        cache.set(cache_token_key, '{}'.format(token), token_timeout_seconds)
        
        # Encrypt the token and send email 
        self.token = make_password(str(token))
        super(VerificationCode, self).save(*args, **kwargs)

        

    def validate(self, token: str=None, device_uuid: str=None):
        if not token or not self.token:
            return False
        
        if self.device:
            if not device_uuid or self.device.uuid != device_uuid:
                return False 

        if (timezone.now() - self.created).total_seconds() > token_timeout_seconds:
            # code expired delete it.
            self.delete()
            return False
            
        return check_password(token, self.token)

TEMPLATES_USES = (
    ('verification_code', _("Verification code: send each time a code is generated")),
    ('other', _("Other: Nets core not implemented use. You can use this from any view quering from nets_core.models.EmailTemplate"))
)
class EmailTemplate(OwnedModel):
    name = models.CharField(_("Name"), max_length=150, unique=True)
    html_body = models.TextField(_("HTML BODY"), help_text=_("You can use Django template language see: https://docs.djangoproject.com/en/4.1/ref/templates/language/ "))
    text_body = models.TextField(_("TXT BODY"), help_text=_("You can use Django template language see: https://docs.djangoproject.com/en/4.1/ref/templates/language/ "))
    enabled = models.BooleanField(_("Enabled?"), default=True)
    use_for = models.CharField(_("Use template for"), default='other', choices=TEMPLATES_USES, max_length=50)
    project_content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE, null=True, blank=True)
    project_id = models.PositiveIntegerField(null=True, blank=True)
    project = GenericForeignKey('project_content_type', 'project_id')
    
    class Meta:
        verbose_name = _("Email Template")
        verbose_name_plural = _("Email Templates")
        db_table = 'nets_core_email_template'
        indexes = [
            models.Index(fields=['project_content_type', 'project_id'], name='email_template_index')
        ]
    
    def __str__(self) -> str:
        if self.project:
            return f'{self.name} - {self.project}'
        return self.name




class CustomEmail(OwnedModel):
    subject = models.CharField(_("subject"), max_length=250)
    to_email = models.TextField(_("To Email"), max_length=250)
    from_email = models.CharField(_("From Email"), max_length=250)
    html_body = models.TextField(_("HTML Body"))
    txt_body = models.TextField(_("TXT Body"), null=True, blank=True)
    completed = models.BooleanField(_("completed"), default=False)
    sent_count = models.IntegerField(_("Sent count"), default=0)
    failed_count = models.IntegerField(_("Failed count"), default=0)
    project_content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE, null=True, blank=True)
    project_id = models.PositiveIntegerField(null=True, blank=True)
    project = GenericForeignKey('project_content_type', 'project_id')
    

    class Meta:
        verbose_name = _("Email")
        verbose_name_plural = _("Emails")
        db_table = 'nets_core_custom_email'
        indexes = [
            models.Index(fields=['project_content_type', 'project_id'], name='custom_email_index')
        ]

    def __str__(self):
        return self.subject

class EmailNotification(models.Model):
    subject = models.CharField(_("subject"), max_length=250)
    to = models.TextField(_("To Email"), max_length=250)
    from_email = models.CharField(_("From Email"), max_length=250)
    body = models.TextField(_("HTML Body"))
    txt_body = models.TextField(_("TXT Body"), null=True, blank=True)
    sent = models.BooleanField(_("sent"), default=False)
    tries = models.IntegerField(_("tries"), default=0)
    sent_at = models.DateTimeField(_("Sent at"), null=True)
    created = models.DateTimeField(_("created"), auto_now_add=True)
    updated = models.DateTimeField(_("created"), auto_now=True)
    custom_email = models.ForeignKey(
        CustomEmail, null=True, on_delete=models.CASCADE)
    project_content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE, null=True, blank=True)
    project_id = models.PositiveIntegerField(null=True, blank=True)
    project = GenericForeignKey('project_content_type', 'project_id')

    def __str__(self):
        if self.project:
            return "%s %s %s" % (self.to, self.subject, self.project)
        
        return "%s %s" % (self.to, self.subject)

    class Meta:
        verbose_name = _("Email Notification")
        verbose_name_plural = _("Email Notifications")
        db_table = 'nets_core_email_notification'
        indexes = [
            models.Index(fields=['project_content_type', 'project_id'], name='email_notification_index')
        ]
        
        

class UserDevice(OwnedModel):
    uuid = models.UUIDField(_("UUID"), default=uuid4, editable=False, unique=True)
    name = models.CharField(_("Name"), max_length=250)
    os = models.CharField(_("OS"), max_length=250, null=True, blank=True)
    os_version = models.CharField(_("OS Version"), max_length=250, null=True, blank=True)
    app_version = models.CharField(_("App Version"), max_length=250, null=True, blank=True)
    device_type = models.CharField(_("Device Type"), max_length=250, null=True, blank=True)
    device_token = models.CharField(_("Device Token"), max_length=250, null=True, blank=True)
    firebase_token = models.CharField(_("Firebase Token"), max_length=250, null=True, blank=True)
    active = models.BooleanField(_("Active"), default=True)
    last_login = models.DateTimeField(_("Last login"), null=True)
    ip = models.CharField(_("IP"), max_length=250, null=True, blank=True)

    class Meta:
        verbose_name = _("User Device")
        verbose_name_plural = _("User Devices")
        db_table = 'nets_core_user_device'


    def __str__(self):
        return self.name
    
    def save(self, *args, **kwargs):
            
        super(UserDevice, self).save(*args, **kwargs)

class UserFirebaseNotification(OwnedModel):
    message = models.TextField(_("Message"), null=True, blank=True)
    devices = models.ManyToManyField(UserDevice, related_name='notifications')
    data = models.JSONField(_("Data"), null=True, blank=True)
    sent = models.BooleanField(_("Sent"), default=False)
    message_id = models.CharField(_("Message ID"), max_length=250, null=True, blank=True)
    error = models.TextField(_("Error"), null=True, blank=True)
    device = models.ForeignKey(UserDevice, on_delete=models.CASCADE)

    class Meta:
        verbose_name = _("User Firebase Notification")
        verbose_name_plural = _("User Firebase Notifications")
        db_table = 'nets_core_firebase_notification'

    def __str__(self):
        return self.message_id


from django.apps import AppConfig


class UsersConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'nets_core'
    
    def ready(self):
        import nets_core.listeners


from datetime import datetime
from pathlib import Path, PosixPath
from django.core.management.base import BaseCommand
from django.conf import settings
import os
import json
import sys
import importlib
import re

class Command(BaseCommand):
    # use stdout and styles to print messages
    help="""
        This command will check if required settings are present in the settings.py file.
        and check if addtional files are present and configured correctly.
        
        if --create flag is passed, it will create the missing files and add the required settings.
        
        for settings.py file, it will check for the following settings:
        oauth2_provider, django_celery_beat, corsheaders,  nets_core in INSTALLED_APPS
        nets-core.auth_urls in ROOT_URLCONF
        corsheaders.middleware.CorsMiddleware in MIDDLEWARE
        oauth2_provider.middleware.OAuth2TokenMiddleware in MIDDLEWARE        
        
        site variables:
        SITE_DOMAIN = ''
        SITE_NAME = ''
        SITE_LOGO = ''
        SITE_LOGO_WHITE = ''
        SITE_LOGO_BLACK = ''
        SITE_DESCRIPTION = ''
        
        celery settings:
            this should be present in settings.py if --create flag is passed default settings will be added using redis as broker and result backend. 
            Please make sure redis is installed and running. and /0 is the default database. if you have others projects using redis, please change the database number to avoid conflicts.
            CELERY_BROKER_URL = 'redis://localhost:6379/0'
            CELERY_RESULT_BACKEND = 'redis://localhost:6379/0'
            CELERY_ACCEPT_CONTENT = ['application/json']
            CELERY_RESULT_SERIALIZER = 'json'
            CELERY_TIMEZONE = TIME_ZONE
            CELERY_TASK_SERIALIZER = 'json'
        
        will add read of the environment variable DJANGO_ENV and add it to the settings.py file.
        ENV = os.getenv('DJANGO_ENV')
        will add check for settings_[ENV].py and import it if it exists.
        if ENV:
            try:
                from .settings_{} import *
            except ImportError:
                pass
        
        
        celery
        Add required settings for nets-core to settings.py 
        create a celery.py file in the project root directory.
    """
    
    REQUIRE_SETTINGS = [
        'INSTALLED_APPS',
        'ROOT_URLCONF',
        'MIDDLEWARE',
        'SITE_DOMAIN',
        'SITE_NAME',
        'CELERY_BROKER_URL',
        'CELERY_RESULT_BACKEND',
        'CELERY_ACCEPT_CONTENT',
        'CELERY_RESULT_SERIALIZER',
        'CELERY_TIMEZONE',
        'CELERY_TASK_SERIALIZER',
        'ENV',
        'CACHES',
        'ASGI_APPLICATION',
        'CHANNEL_LAYERS',
    ]
    
    def add_arguments(self, parser):
        parser.add_argument('--create', action='store_true', help='Create missing files and add required settings to settings.py')
        parser.add_argument('--force', action='store_true', help='Force create missing files and add required settings to settings.py')
        parser.add_argument('--site-domain', type=str, help='Site domain')
        parser.add_argument('--site-name', type=str, help='Site name')
        
    def handle(self, *args, **options):
        create = options.get('create')
        force = options.get('force')
        site_domain = options.get('site_domain')
        site_name = options.get('site_name')
        
        if not site_domain:
            site_domain = input('Enter the site domain: ')
        
        if not site_name:
            site_name = input('Enter the site name: ')
        
        if create:
            self.create_files(site_domain, site_name, force=force)
        
        else:
            self.check_settings(site_domain, site_name)
            
            
    def check_settings(self, site_domain, site_name):
        # check if the required settings are present in the settings.py file.
        # and check if addtional files are present and configured correctly.
        from django.conf import settings
        base_dir = settings.BASE_DIR
        
        
        
        # check for installed apps
        
        try:
            installed_apps = settings.INSTALLED_APPS
            if 'oauth2_provider' not in installed_apps:
                self.stdout.write(self.style.ERROR('oauth2_provider not found in INSTALLED_APPS in settings.py file.'))            
            
            
            if 'django_celery_beat' not in installed_apps:
                self.stdout.write(self.style.ERROR('django_celery_beat not found in INSTALLED_APPS in settings.py file.'))            
            
            
            if 'corsheaders' not in installed_apps:
                self.stdout.write(self.style.ERROR('corsheaders not found in INSTALLED_APPS in settings.py file.'))
            
            if 'nets_core' not in installed_apps:
                self.stdout.write(self.style.ERROR('nets_core not found in INSTALLED_APPS in settings.py file.'))
        except Exception as e:
            self.stdout.write(self.style.ERROR('INSTALLED_APPS not found in settings.py file.'))
            pass
        
        
        # check for ROOT_URLCONF
                       
        if not hasattr(settings, 'ROOT_URLCONF'):
            self.stdout.write(self.style.ERROR('ROOT_URLCONF not found in settings.py file.'))
            pass
        # check for MIDDLEWARE
        
        try:
            middleware = settings.MIDDLEWARE
            if 'corsheaders.middleware.CorsMiddleware' not in middleware:
                self.stdout.write(self.style.ERROR('corsheaders.middleware.CorsMiddleware not found in MIDDLEWARE in settings.py file.'))
            
            if 'oauth2_provider.middleware.OAuth2TokenMiddleware' not in middleware:
                self.stdout.write(self.style.ERROR('oauth2_provider.middleware.OAuth2TokenMiddleware not found in MIDDLEWARE in settings.py file.'))
            
        except Exception as e:
            self.stdout.write(self.style.ERROR('MIDDLEWARE not found in settings.py file.'))
            pass
        
        try:
            site_domain = settings.SITE_DOMAIN
            site_name = settings.SITE_NAME
        except Exception as e:
            self.stdout.write(self.style.ERROR('SITE_DOMAIN or SITE_NAME not found in settings.py file.'))
            pass
        
        try:
            celery_broker_url = settings.CELERY_BROKER_URL
            celery_result_backend = settings.CELERY_RESULT_BACKEND
            celery_accept_content = settings.CELERY_ACCEPT_CONTENT
            celery_result_serializer = settings.CELERY_RESULT_SERIALIZER
            celery_timezone = settings.CELERY_TIMEZONE
            celery_task_serializer = settings.CELERY_TASK_SERIALIZER
        except Exception as e:
            self.stdout.write(self.style.ERROR('Celery settings not found in settings.py file.'))
            pass
        
        try:
            env = settings.ENV
        except Exception as e:
            self.stdout.write(self.style.NOTICE('ENV not found in settings.py file. This is optional.'))
            pass
            
            
    def create_files(self, site_domain, site_name, force=False):
        time = datetime.now().strftime('%Y%m%d%H%M%S')
        # create the missing files and add the required settings.
        from django.conf import settings as django_settings
        base_dir = django_settings.BASE_DIR
        
        manage_py = os.path.join(base_dir, 'manage.py')
        if not os.path.exists(manage_py):
            self.stdout.write(self.style.ERROR('manage.py not found.'))
            sys.exit(1)
        
        # get  os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'nets_core.settings') from manage.py
        try:
            with open(manage_py, 'r') as f:
                manage_py_content = f.read()
                settings_module_env = re.findall(r"os.environ.setdefault\('DJANGO_SETTINGS_MODULE', '(.*)'\)", manage_py_content)
                self.stdout.write(settings_module_env[0])
                project_name = settings_module_env[0].split('.')[0]
                if not os.path.exists(os.path.join(base_dir, f'{project_name}/settings.py')):
                    self.stdout.write(self.style.ERROR(f'{project_name}/settings.py not found.'))
                    sys.exit(1)
                else:
                    settings = importlib.import_module(f'{project_name}.settings')
                    
                

        except Exception as e:
            self.stdout.write(self.style.ERROR(e))
            self.stdout.write(self.style.ERROR('Error reading manage.py file. Please run the command from the project root directory.'))
            sys.exit(1)
            
        # check for installed apps
        if not 'daphne' in settings.INSTALLED_APPS:
            self.stdout.write(self.style.NOTICE('Adding daphne to INSTALLED_APPS in settings.py file.'))
            settings.INSTALLED_APPS = ['daphne'] + settings.INSTALLED_APPS
            
        if 'oauth2_provider' not in settings.INSTALLED_APPS:
            self.stdout.write(self.style.NOTICE('Adding oauth2_provider to INSTALLED_APPS in settings.py file.'))
            settings.INSTALLED_APPS.append('oauth2_provider')
        
        if 'django_celery_beat' not in settings.INSTALLED_APPS:
            self.stdout.write(self.style.NOTICE('Adding django_celery_beat to INSTALLED_APPS in settings.py file.'))
            settings.INSTALLED_APPS.append('django_celery_beat')
            
        if 'corsheaders' not in settings.INSTALLED_APPS:
            self.stdout.write(self.style.NOTICE('Adding corsheaders to INSTALLED_APPS in settings.py file.'))
            settings.INSTALLED_APPS.append('corsheaders')
            
        if 'nets_core' not in settings.INSTALLED_APPS:
            self.stdout.write(self.style.NOTICE('Adding nets_core to INSTALLED_APPS in settings.py file.'))
            settings.INSTALLED_APPS.append('nets_core')
            
        # check for ROOT_URLCONF
        if not hasattr(settings, 'ROOT_URLCONF'):
            self.stdout.write(self.style.NOTICE('Adding ROOT_URLCONF to settings.py file.'))
            settings.ROOT_URLCONF = f'{project_name}.urls'
            
        # check for MIDDLEWARE
        if 'corsheaders.middleware.CorsMiddleware' not in settings.MIDDLEWARE:
            self.stdout.write(self.style.NOTICE('Adding corsheaders.middleware.CorsMiddleware to MIDDLEWARE in settings.py file.'))
            settings.MIDDLEWARE.append('corsheaders.middleware.CorsMiddleware')
            
        if 'oauth2_provider.middleware.OAuth2TokenMiddleware' not in settings.MIDDLEWARE:
            self.stdout.write(self.style.NOTICE('Adding oauth2_provider.middleware.OAuth2TokenMiddleware to MIDDLEWARE in settings.py file.'))
            settings.MIDDLEWARE.append('oauth2_provider.middleware.OAuth2TokenMiddleware')
            
        # check for site variables
        if not hasattr(settings, 'SITE_DOMAIN'):
            self.stdout.write(self.style.NOTICE('Adding SITE_DOMAIN to settings.py file.'))
            settings.SITE_DOMAIN = site_domain
            
        if not hasattr(settings, 'SITE_NAME'):
            self.stdout.write(self.style.NOTICE('Adding SITE_NAME to settings.py file.'))
            settings.SITE_NAME = site_name
            
        if not hasattr(settings, 'SITE_LOGO'):
            self.stdout.write(self.style.NOTICE('Adding SITE_LOGO to settings.py file.'))
            settings.SITE_LOGO = ''
            
        if not hasattr(settings, 'SITE_LOGO_WHITE'):
            self.stdout.write(self.style.NOTICE('Adding SITE_LOGO_WHITE to settings.py file.'))
            settings.SITE_LOGO_WHITE = ''
            
        if not hasattr(settings, 'SITE_LOGO_BLACK'):
            self.stdout.write(self.style.NOTICE('Adding SITE_LOGO_BLACK to settings.py file.'))
            settings.SITE_LOGO_BLACK = ''
            
        if not hasattr(settings, 'SITE_DESCRIPTION'):
            self.stdout.write(self.style.NOTICE('Adding SITE_DESCRIPTION to settings.py file.'))
            settings.SITE_DESCRIPTION = ''
            
        # check for celery settings
        if not hasattr(settings, 'CELERY_BROKER_URL'):
            self.stdout.write(self.style.NOTICE('Adding CELERY_BROKER_URL to settings.py file.'))
            settings.CELERY_BROKER_URL = 'redis://localhost:6379/0'
        
        if not hasattr(settings, 'CELERY_RESULT_BACKEND'):
            self.stdout.write(self.style.NOTICE('Adding CELERY_RESULT_BACKEND to settings.py file.'))
            settings.CELERY_RESULT_BACKEND = 'redis://localhost:6379/0'
            
        if not hasattr(settings, 'CELERY_ACCEPT_CONTENT'):
            self.stdout.write(self.style.NOTICE('Adding CELERY_ACCEPT_CONTENT to settings.py file.'))
            settings.CELERY_ACCEPT_CONTENT = ['application/json']
            
        if not hasattr(settings, 'CELERY_RESULT_SERIALIZER'):
            self.stdout.write(self.style.NOTICE('Adding CELERY_RESULT_SERIALIZER to settings.py file.'))
            settings.CELERY_RESULT_SERIALIZER = 'json'
            
        if not hasattr(settings, 'CELERY_TIMEZONE'):
            self.stdout.write(self.style.NOTICE('Adding CELERY_TIMEZONE to settings.py file.'))
            settings.CELERY_TIMEZONE = settings.TIME_ZONE
            
        if not hasattr(settings, 'CELERY_TASK_SERIALIZER'):
            self.stdout.write(self.style.NOTICE('Adding CELERY_TASK_SERIALIZER to settings.py file.'))
            settings.CELERY_TASK_SERIALIZER = 'json'
            
        # check for ENV
        if not hasattr(settings, 'ENV'):
            self.stdout.write(self.style.NOTICE('Adding ENV to settings.py file.'))
            settings.ENV = os.getenv('DJANGO_ENV')
            
        # check for settings_[ENV].py
        if settings.ENV:
            if not os.path.exists(os.path.join(base_dir, f'{project_name}/settings_{settings.ENV}.py')):
                self.stdout.write(self.style.WARNING(f'{project_name}/settings_{settings.ENV}.py not found.'))
                
        #     CACHES = {
        #     'default': {
        #     'BACKEND': 'django.core.cache.backends.memcached.PyMemcacheCache',
        #     'LOCATION': '127.0.0.1:11211'
        #     }
        # }
        # check for cache settings
        if not hasattr(settings, 'CACHES'):
            self.stdout.write(self.style.NOTICE('Adding CACHES to settings.py file.'))
            settings.CACHES = {
                'default': {
                    'BACKEND': 'django.core.cache.backends.memcached.PyMemcacheCache',
                    'LOCATION': '127.0.0.1:11211'
                }
            }
        
        if not hasattr(settings, 'ASGI_APPLICATION'):
            self.stdout.write(self.style.NOTICE('Adding ASGI_APPLICATION to settings.py file.'))
            settings.ASGI_APPLICATION = f'{project_name}.asgi.application'
            
        if not hasattr(settings, 'CHANNEL_LAYERS'):
            #             CHANNEL_LAYERS = {
            #     'default': {
            #         'BACKEND': 'channels_redis.core.RedisChannelLayer',
            #         'CONFIG': {
            #             "hosts": [('127.0.0.1', 6379)],
            #             "capacity": 1500,
            #             "expiry": 10
            #         },
            #     },
            #     'dev': {
            #         "BACKEND": "channels.layers.InMemoryChannelLayer"
            #     }
            # }
            self.stdout.write(self.style.NOTICE('Adding CHANNEL_LAYERS to settings.py file.'))
            settings.CHANNEL_LAYERS = {
                'default': {
                    'BACKEND': 'channels_redis.core.RedisChannelLayer',
                    'CONFIG': {
                        "hosts": [('127.0.0.1', 6379)],
                        "capacity": 1500,
                        "expiry": 10
                    },
                },
                'dev': {
                    "BACKEND": "channels.layers.InMemoryChannelLayer"
                }
            }           
                                   
        
        # create celery.py file
        celery_py = os.path.join(base_dir, f'{project_name}/celery.py')
        if not os.path.exists(celery_py) or force: 
            self.stdout.write(self.style.NOTICE('Creating celery.py file.'))
            with open(celery_py, 'w') as f:
                f.write(f"import os\n\n")
                f.write(f"from celery import Celery\n\n")
                f.write(f"os.environ.setdefault('DJANGO_SETTINGS_MODULE', '{project_name}.settings')\n\n")
                f.write(f"app = Celery('{project_name}')\n\n")
                f.write(f"app.config_from_object('django.conf:settings', namespace='CELERY')\n\n")
                f.write(f"app.autodiscover_tasks()\n")
        else:
            self.stdout.write(self.style.NOTICE('celery.py file already exists.'))
        
        # create asgi.py file
        asgi_py = os.path.join(base_dir, f'{project_name}/asgi.py')
        if not os.path.exists(asgi_py) or force:
            if os.path.exists(asgi_py) and force:
                # backup the file
                self.stdout.write(self.style.NOTICE(f'Backing up {asgi_py} file.'))
                
                os.rename(asgi_py, f'{asgi_py}_{time}.bak')
                
            self.stdout.write(self.style.NOTICE('Creating asgi.py file.'))
            with open(asgi_py, 'w') as f:
                f.write(f"\"\"\"\n")
                f.write(f"ASGI config for {project_name} project.\n\n")
                f.write(f"It exposes the ASGI callable as a module-level variable named ``application``.\n\n")
                f.write(f"For more information on this file, see\n")
                f.write(f"https://docs.djangoproject.com/en/4.1/howto/deployment/asgi/\n")
                f.write(f"\"\"\"\n\n")
                f.write(f"import os\n\n")
                f.write(f"from django.core.asgi import get_asgi_application\n")
                f.write(f"from nets_core.middleware.auth_token import AuthTokenMiddleware\n")
                f.write(f"from channels.routing import ProtocolTypeRouter, URLRouter\n\n")
                f.write(f"os.environ.setdefault('DJANGO_SETTINGS_MODULE', '{project_name}.settings')\n\n")
                f.write(f"django_asgi_application = get_asgi_application()\n\n")
                f.write(f"application = ProtocolTypeRouter({{\n")
                f.write(f"    \"http\": django_asgi_application,\n")
                f.write(f"    #\"websocket\": AuthTokenMiddleware(\n")
                f.write(f"        # URLRouter(\n")
                f.write(f"            # Just like the Django URL router, but for WebSockets\n")
                f.write(f"            # \"nets_core.routing.websocket_urlpatterns\"\n")
                f.write(f"    #   )\n")
                f.write(f"    # ),\n")
                f.write(f"}})\n")
                
        else:
            self.stdout.write(self.style.NOTICE('asgi.py file already exists.'))
            
        # create settings_[ENV].py file
        
        settings_env_py = os.path.join(base_dir, f'{project_name}/settings_nets.py')
        if not os.path.exists(settings_env_py) or force:
            if os.path.exists(settings_env_py)  and force:
                # backup the file
                self.stdout.write(self.style.NOTICE(f'Backing up {settings_env_py} file.'))
                
                os.rename(settings_env_py, f'{settings_env_py}_{time}.bak')
                    
            self.stdout.write(self.style.NOTICE(f'Creating settings_nets.py file.'))
            settings_content = ""
            for attr in dir(settings):
                if not attr in self.REQUIRE_SETTINGS:
                    continue
                if not attr.startswith('__'):
                    value = getattr(settings, attr)
                    if isinstance(value, str):
                        settings_content += f"{attr} = '{value}'\n"
                    elif isinstance(value, bool):
                        settings_content += f"{attr} = {value}\n"
                    elif isinstance(value, int):
                        settings_content += f"{attr} = {value}\n"
                    elif isinstance(value, list):
                        settings_content += f"{attr} = [\n"
                        for item in value:
                            settings_content += f"    '{item}',\n"
                        settings_content += "]\n"
                    elif isinstance(value, dict):
                        settings_content += f"{attr} = {json.dumps(value, indent=4, cls=SettingsEncoder)}\n"
                    elif isinstance(value, datetime):
                        settings_content += f"{attr} = datetime.datetime({value.year}, {value.month}, {value.day}, {value.hour}, {value.minute}, {value.second}, {value.microsecond})\n"
                        
            with open(settings_env_py, 'w') as f:
                f.write(settings_content)
        else:
            self.stdout.write(self.style.NOTICE(f'settings_nets.py file already exists.'))
            
        # append to settings file
        # check if setings_nets.py is already imported
        with open(os.path.join(base_dir, f'{project_name}/settings.py'), 'r') as f:
            settings_content = f.read()
            if 'from .settings_nets import *' in settings_content:
                self.stdout.write(self.style.SUCCESS(f'settings_nets.py already imported in settings.py file.'))
                return
            
        with open(os.path.join(base_dir, f'{project_name}/settings.py'), 'a') as f:
            f.write(f"\n\ntry:\n")
            f.write(f"    from .settings_nets import *\n")
            f.write(f"except ImportError:\n")
            f.write(f"    pass\n")

class SettingsEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime):
            return obj.isoformat()
        if isinstance(obj, set):
            return list(obj)
        if isinstance(obj, Path):
            return str(obj)
        if isinstance(obj, type):
            return str(obj)
        if isinstance(obj, PosixPath):
            return str(obj)
        return json.JSONEncoder.default(self, obj)


import json
import logging
from collections import namedtuple

from django.http.response import JsonResponse

from nets_core.params import RequestParam
from nets_core.responses import permission_denied
from django.utils.translation import gettext_lazy as _

logger = logging.getLogger(__name__)

def get_value_from_data_key(data, key, params, customer, files):
    # Get value from data calling parse_param
    # to validate type and validate if is required
    # additional add customer to RequestParam instances
    # to ensure validation
    
    try:
        value = data[key]
        if not key in params.keys():
            return value
    except KeyError:
        pass
    
    param_type = params[key]
    if isinstance(param_type, RequestParam):
        if customer:
            # Append customer to parameter to validate
            # is required by RequestParam
            param_type.customer = customer
        if param_type.type == 'file':
            
            return param_type.get_file(files)
        
        return param_type.get_value(data)
        
    value = parse_param(data, [key, param_type])
    return value   

def parse_param(data, k):

    if isinstance(k, str):
        d = data.get(k, None)
        return d

    elif isinstance(k, list):
        data_key = k[0]
        data_type = k[1]

        if not isinstance(data_type, RequestParam):
            # build RequestParam object
            data_type  = RequestParam(data_key, data_type)
        
        # Exceptions handle by request_params_handler
        value = data_type.get_value(data)
        if not value and data_type.optional:
            print("returning default", data_type.default)
            return None
            
        return data_type.get_value(data)



def extract_data(request):
    
    if request.method == 'GET':
        return request.GET

    data = request.POST

    content_type = request.META.get(
        'CONTENT_TYPE', 'application/x-www-form-urlencoded')
    
    if content_type.startswith('application/json'):
        data = json.loads(request.body)

    return data

def request_params_handler(request, params: dict|list={}):
    data = extract_data(request)

    if request.user.is_anonymous and not request.public:
        return permission_denied()
    # TODO: Add support for multi customer projects
    customer = None
    # customer_id = data.get('customer_id', None)
    # if customer_id:
    #     customer = get_customer_from_db(data['customer_id'])
    # if request.customer_required and not customer:
    #     return JsonResponse({"res": 0, "message": "customer_id is required"}, status=400)
        
    # params_keys = params.keys()
    # request.customer = customer
    parsed_data = {}
    # found_params = []

    for k in data:
        # parse data from request
        try:
            parsed_data[k] = get_value_from_data_key(data, k, params, customer, request.FILES)
        except Exception as e:
            # Error parsing values
            msg = e.__str__()
            return JsonResponse({"res": 0, "message": msg}, status=400)
    
    # extract files
    for k in request.FILES:
        if k in params.keys():
            parsed_data[k] = get_value_from_data_key(data, k, params, customer, request.FILES)

    if 'action' in parsed_data.keys():
        if not 'paginated_by' in parsed_data.keys():
            parsed_data['paginated_by'] = 25
        if not 'page'in parsed_data.keys():
            parsed_data["page"] = 1

    missing_params = []
    
    for p in params:
        optional = False
        if isinstance(params[p], RequestParam):
            optional = params[p].optional
            default = params[p].default if hasattr(params[p], 'default') else None
            if not p in parsed_data.keys() and optional:
                parsed_data[p] = default

        if p not in parsed_data.keys() and not optional:
            # Omit missing if is instance of RequestParams
            # because this class check and validate if self
            missing_params.append(p)

    if missing_params:
        message = "Missing params: {}".format(", ".join(missing_params))
        return JsonResponse({"res": 0, "message": message}, status=400)

    request.parsed_data = namedtuple(
        "Data", parsed_data.keys())(*parsed_data.values())
    # Keep compatibility with old code
    request.parse_data = request.parsed_data
    request.params = request.parsed_data
    return request


def get_request_obj(request, obj):
    """
        Get object from db using index_field
        and append to request object
        
        if obj has customer field, append customer
        attribute to request

        if obj has owner or user field, append
        is_owner and has_owner attributes to request.
        
        if not perm(public object or permission granted) and
        has_owner and not is_owner, return 404 

    """
    index_field = request.index_field
    try:
        o_query = {}
        if not hasattr(request.parsed_data, index_field):
            return JsonResponse({"res": 0, "message": _("index_field doesn't sent a valid value")})
        
        index_value = getattr(request.parsed_data, index_field, None)
        if not index_value:
            return JsonResponse({"res": 0, "message": _("index_field doesn't sent a valid value")})
        
        if isinstance(index_value, dict):
            index_value = index_value['value']

        o_query[index_field] = index_value
            
        if hasattr(obj, 'customer'):
            o_query['customer'] = request.customer
        
        o = obj.objects.get(**o_query)
        request.obj = o
        request.is_owner = False
        request.has_owner = False
        if hasattr(o, 'owner'):
            request.is_owner = o.owner == request.user
            request.has_owner = True

        if hasattr(o, 'user'):
            request.is_owner = o.user == request.user
            request.has_owner = True
            
        if not request.public:
            if not request.perm and (request.has_owner and not request.is_owner):
                logger.warning(f"User {request.user.id} tried to access {request.obj} {request.obj.id} without permission")
                return JsonResponse({"res": 0, "message": _("not found")}, status=404)
            
        request.obj = o
        return request
    except obj.DoesNotExist:
        return JsonResponse({"res": 0, "message": _("not found")}, status=404)


from celery import shared_task
# get_user_model() returns the User model that is active in this project
from django.contrib.auth import get_user_model
from django.conf import settings

User = get_user_model()


@shared_task
def send_user_devices_notifications(user_id:  int, title: str, message: str, data: dict, channel: str=None):
    from nets_core.firebase_messages import send_user_device_notification
    user = User.objects.get(id=user_id)    
    # ensure data is a dict of strings
    data = {k: str(v) for k, v in data.items()}
    send_user_device_notification(user, title, message, data, channel)

@shared_task
def check_permissions(user_id: int, permission: str):
    user = User.objects.get(id=user_id)
    return user.has_perm(permission)


from django.contrib import admin
from nets_core import models




@admin.register(models.Role)
class RoleAdmin(admin.ModelAdmin):
    model = models.Role
    
    list_display = ('name', 'description', 'project', 'enabled')
    list_filter = ('project', 'enabled')
    search_fields = ('name', 'description', 'project')
        
    
@admin.register(models.Permission)
class PermissionAdmin(admin.ModelAdmin):
    model = models.Permission
    
    list_display = ('codename', 'description', 'project')
    list_filter = ('project',)
    search_fields = ('codename', 'description', 'project')
    
    def get_queryset(self, request):
        qs = super().get_queryset(request)
        if request.user.is_superuser:
            return qs
        return qs.filter(roles__user=request.user)
    
    
@admin.register(models.EmailNotification)
class EmailNotificationAdmin(admin.ModelAdmin):
    model = models.EmailNotification
    
    list_display = ('subject', 'to', 'custom_email', 'sent', 'tries', 'created', 'project')
    list_filter = ('created', 'sent', 'project')
    
    search_fields = ('subject', 'email', 'project')
    
    def get_queryset(self, request):
        qs = super().get_queryset(request)
        if request.user.is_superuser:
            return qs
        return qs.filter(email=request.user.email)
    
@admin.register(models.CustomEmail)
class CustomEmailAdmin(admin.ModelAdmin):
    model = models.CustomEmail
    
    list_display = ('subject', 'to_email', 'project', 'completed', 'sent_count', 'failed_count')
    list_filter = ('project', 'completed')
    search_fields = ('subject', 'project')
    
    def get_queryset(self, request):
        qs = super().get_queryset(request)
        if request.user.is_superuser:
            return qs
        return qs.filter(project=request.user.project)


@admin.register(models.EmailTemplate)
class EmailTemplateAdmin(admin.ModelAdmin):
    model = models.EmailTemplate
    
    list_display = ('name', 'project', 'use_for', 'enabled')
    list_filter = ('project', 'enabled')
    search_fields = ('name', 'project', 'use_for')
    
    def get_queryset(self, request):
        qs = super().get_queryset(request)
        if request.user.is_superuser:
            return qs
        return qs.filter(project=request.user.project)
    
    def save_model(self, request, obj, form, change):
        if not obj.user:
            obj.user = request.user
        obj.save()


# Generated by Django 5.0.4 on 2024-05-05 19:06

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('nets_core', '0009_userdevice_ip'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Permission',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150, verbose_name='Name')),
                ('codename', models.CharField(max_length=150, verbose_name='Codename')),
                ('description', models.CharField(blank=True, max_length=250, null=True, verbose_name='Description')),
                ('project_id', models.PositiveIntegerField(blank=True, null=True)),
            ],
            options={
                'verbose_name': 'Permission',
                'verbose_name_plural': 'Permissions',
                'db_table': 'nets_core_permission',
            },
        ),
        migrations.CreateModel(
            name='Role',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150, verbose_name='Name')),
                ('codename', models.CharField(max_length=150, verbose_name='Codename')),
                ('description', models.CharField(max_length=250, verbose_name='Description')),
                ('project_id', models.PositiveIntegerField(blank=True, null=True)),
            ],
            options={
                'verbose_name': 'Role',
                'verbose_name_plural': 'Roles',
                'db_table': 'nets_core_role',
            },
        ),
        migrations.CreateModel(
            name='UserRole',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('project_id', models.PositiveIntegerField(blank=True, null=True)),
            ],
            options={
                'verbose_name': 'User Role',
                'verbose_name_plural': 'User Roles',
                'db_table': 'nets_core_user_role',
            },
        ),
        migrations.AlterModelOptions(
            name='emailnotification',
            options={'verbose_name': 'Email Notification', 'verbose_name_plural': 'Email Notifications'},
        ),
        migrations.AlterModelOptions(
            name='emailtemplate',
            options={'verbose_name': 'Email Template', 'verbose_name_plural': 'Email Templates'},
        ),
        migrations.AddField(
            model_name='customemail',
            name='project_content_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype'),
        ),
        migrations.AddField(
            model_name='customemail',
            name='project_id',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='emailnotification',
            name='project_content_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype'),
        ),
        migrations.AddField(
            model_name='emailnotification',
            name='project_id',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='emailtemplate',
            name='project_content_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype'),
        ),
        migrations.AddField(
            model_name='emailtemplate',
            name='project_id',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AddIndex(
            model_name='customemail',
            index=models.Index(fields=['project_content_type', 'project_id'], name='custom_email_index'),
        ),
        migrations.AddIndex(
            model_name='emailnotification',
            index=models.Index(fields=['project_content_type', 'project_id'], name='email_notification_index'),
        ),
        migrations.AddIndex(
            model_name='emailtemplate',
            index=models.Index(fields=['project_content_type', 'project_id'], name='email_template_index'),
        ),
        migrations.AlterModelTable(
            name='customemail',
            table='nets_core_custom_email',
        ),
        migrations.AlterModelTable(
            name='emailnotification',
            table='nets_core_email_notification',
        ),
        migrations.AlterModelTable(
            name='emailtemplate',
            table='nets_core_email_template',
        ),
        migrations.AddField(
            model_name='permission',
            name='project_content_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='nets_core_permissions', to='contenttypes.contenttype'),
        ),
        migrations.AddField(
            model_name='role',
            name='permissions',
            field=models.ManyToManyField(related_name='roles', to='nets_core.permission'),
        ),
        migrations.AddField(
            model_name='role',
            name='project_content_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype'),
        ),
        migrations.AddField(
            model_name='userrole',
            name='project_content_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype'),
        ),
        migrations.AddField(
            model_name='userrole',
            name='role',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='nets_core.role'),
        ),
        migrations.AddField(
            model_name='userrole',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddIndex(
            model_name='permission',
            index=models.Index(fields=['project_content_type', 'project_id'], name='permission_index'),
        ),
        migrations.AddIndex(
            model_name='role',
            index=models.Index(fields=['project_content_type', 'project_id'], name='role_index'),
        ),
    ]




from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django.template import TemplateDoesNotExist, TemplateSyntaxError, Template
from django.template.loader import render_to_string
from django.core.mail import EmailMultiAlternatives

from nets_core.models import EmailNotification

from django.utils.module_loading import import_string
from django.core.mail.backends.smtp import EmailBackend

exclude_domains = []
try:
    exclude_domains = settings.NETS_CORE_EMAIL_EXCLUDE_DOMAINS
except:
    pass

footer_enabled = True
try:
    footer_enabled = settings.NETS_CORE_EMAIL_FOOTER_ENABLED
except:
    pass

footer = "<span><i class=\"small\">{}</i></span>".format(
        _("NETS CORE footer email, you can customize this message in settings.NETS_CORE_EMAIL_FOOTER"))
try:
    footer = f"<span><i class=\"small\">{settings.NETS_CORE_EMAIL_FOOTER}</i></span>"
except:
    pass

footer_template = None
try:
    footer_template = settings.NETS_CORE_EMAIL_FOOTER_TEMPLATE
except:
    pass

try:
    SmsService = import_string(settings.NOTIFY['SMS_SERVICE'])
    sms = SmsService()
except:
    sms = None

mail_debug_enabled = False
try:
    mail_debug_enabled = settings.NETS_CORE_EMAIL_DEBUG_ENABLED
except:
    pass


def send_email(subject: str, email: str|list[str], template: str, context: dict, 
    txt_template: str = None, to_queued: bool = True, force=False, html: str =None):
    """
        Create a email to be sent by command line ./manage.py send_emails
        or dispatch if to_queued is set to False
    """

    if settings.DEBUG and not mail_debug_enabled and not force:
        return (False, _("emails are disabled while debug is true in settings"))

    # Exclude emails from exclude_domain
    if(isinstance(email, str)):
        email_domain = email.split('@')
        if email_domain in exclude_domains:
            return (False, f'{_("Email domain is in NETS_CORE_EMAIL_EXCLUDE_DOMAINS ")} {email_domain}')

    elif isinstance(email, list):
        for e in email:
            email_domain = e.split('@')
            if email_domain in exclude_domains:
                email.pop(email.index(e))
                print(f"{_('Email domain is in NETS_CORE_EMAIL_EXCLUDE_DOMAINS')} {email_domain}")
    
    if not email:
        return (False, _("email is empty"))
    
    full_context = context
    # full_context.update(context)
    content_txt = None
    if template:
        try:
            content_html = render_to_string(template, context=full_context)
            
            if footer_enabled:
                content_html = brandmark_template(content_html)

        except TemplateDoesNotExist:
            print("Template {} does not exist".format(template))
            return (False, _("Template does not exist"))
        except TemplateSyntaxError as e:
            print(e)
            return (False, _("Template syntax error"))
    elif html:
        # render with html
        template = Template(html)
        content_html = template.render(full_context)
        
    else:
        return (False, _("template or html content for send_email is required"))

    params = {
        'to': email,
        'body': content_html,
        'from_email': settings.DEFAULT_FROM_EMAIL,
        'subject': subject,
    }

    if txt_template:
        content_txt = render_to_string(txt_template)
    
    if to_queued:

        if content_txt:
            params['txt_body'] = content_txt
        email_noti = EmailNotification.objects.create(**params)
        email_noti.save()

        return (True, _("Email in queue."))

    else:
        msg = EmailMultiAlternatives(**params)
        msg = EmailMultiAlternatives(
            subject, content_html, settings.DEFAULT_FROM_EMAIL, email)
        msg.content_subtype = "html"
        if content_txt:
            msg.attach_alternative(content_txt, "text/plain")
        result = msg.send(fail_silently=False)
        if result:
            return (True, _("Email sent"))
        else:
            return (False, _("Email wasn't sent"))
    # if sms:
    #     sms.send_sms(phone, text)


def brandmark_template(html):
    if footer_template:
        return brandmark_footer_template(html)

    html = html.replace('</body>', '{}</body>'.format(footer))
    return html


def brandmark_footer_template(html):
    brand = render_to_string(
        footer_template)
    html = html.replace('</body>', '{}</body>'.format(brand))
    return html



# Generated by Django 4.1.7 on 2023-02-22 21:22

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='CustomEmail',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='updated')),
                ('subject', models.CharField(max_length=250, verbose_name='subject')),
                ('to_email', models.TextField(max_length=250, verbose_name='To Email')),
                ('from_email', models.CharField(max_length=250, verbose_name='From Email')),
                ('html_body', models.TextField(verbose_name='HTML Body')),
                ('txt_body', models.TextField(blank=True, null=True, verbose_name='TXT Body')),
                ('completed', models.BooleanField(default=False, verbose_name='completed')),
                ('sent_count', models.IntegerField(default=0, verbose_name='Sent count')),
                ('failed_count', models.IntegerField(default=0, verbose_name='Failed count')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Email',
                'verbose_name_plural': 'Emails',
            },
        ),
        migrations.CreateModel(
            name='VerificationCode',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='updated')),
                ('token', models.CharField(max_length=32, verbose_name='Verification token')),
                ('verified', models.BooleanField(default=False, verbose_name='Verified?')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'db_table': 'nets_core_verification_code',
            },
        ),
        migrations.CreateModel(
            name='EmailTemplate',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='updated')),
                ('name', models.CharField(max_length=150, unique=True, verbose_name='Name')),
                ('html_body', models.TextField(help_text='You can use Django template language see: https://docs.djangoproject.com/en/4.1/ref/templates/language/ ', verbose_name='HTML BODY')),
                ('text_body', models.TextField(help_text='You can use Django template language see: https://docs.djangoproject.com/en/4.1/ref/templates/language/ ', verbose_name='TXT BODY')),
                ('enabled', models.BooleanField(default=True, verbose_name='Enabled?')),
                ('use_for', models.CharField(choices=[('verification_code', 'Verification code: send each time a code is generated'), ('other', 'Other: Nets core not implemented use. You can use this from any view quering from nets_core.models.EmailTemplate')], default='other', max_length=50, verbose_name='Use template for')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='EmailNotification',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('subject', models.CharField(max_length=250, verbose_name='subject')),
                ('to', models.TextField(max_length=250, verbose_name='To Email')),
                ('from_email', models.CharField(max_length=250, verbose_name='From Email')),
                ('body', models.TextField(verbose_name='HTML Body')),
                ('txt_body', models.TextField(blank=True, null=True, verbose_name='TXT Body')),
                ('sent', models.BooleanField(default=False, verbose_name='sent')),
                ('tries', models.IntegerField(default=0, verbose_name='tries')),
                ('sent_at', models.DateTimeField(null=True, verbose_name='Sent at')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='created')),
                ('custom_email', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='nets_core.customemail')),
            ],
        ),
    ]



# Generated by Django 4.1.7 on 2023-08-16 05:08

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import shortuuid.main


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('nets_core', '0002_alter_verificationcode_token'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserDevice',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='updated')),
                ('name', models.CharField(max_length=250, verbose_name='Name')),
                ('os', models.CharField(blank=True, max_length=250, null=True, verbose_name='OS')),
                ('os_version', models.CharField(blank=True, max_length=250, null=True, verbose_name='OS Version')),
                ('app_version', models.CharField(blank=True, max_length=250, null=True, verbose_name='App Version')),
                ('device_id', models.CharField(default=shortuuid.main.ShortUUID.uuid, max_length=250, verbose_name='Device ID')),
                ('device_type', models.CharField(blank=True, max_length=250, null=True, verbose_name='Device Type')),
                ('device_token', models.CharField(blank=True, max_length=250, null=True, verbose_name='Device Token')),
                ('firebase_token', models.CharField(blank=True, max_length=250, null=True, verbose_name='Firebase Token')),
                ('active', models.BooleanField(default=True, verbose_name='Active')),
                ('last_login', models.DateTimeField(null=True, verbose_name='Last login')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'User Device',
                'verbose_name_plural': 'User Devices',
                'db_table': 'nets_core_user_device',
            },
        ),
    ]



from nets_core.firebase_messages import send_user_device_notification
from nets_core.models import VerificationCode
from django.db.models.signals import post_delete, post_save, pre_delete
from django.dispatch import receiver
from nets_core.mail import send_email
from nets_core.models import EmailTemplate
from django.core.cache import cache
from django.utils.translation import gettext_lazy as _
import logging

logger = logging.getLogger(__name__)


@receiver(post_save, sender=VerificationCode)
def send_verification_code_email(sender, instance, created, **kwargs):
    if created:
        # send email
        cache_token_key = instance.get_token_cache_key()
        button_link = {"url": "", "label": cache.get(cache_token_key)}
        template = None
        html = None
        email_template = (
            EmailTemplate.objects.filter(use_for="verification_code", enabled=True)
            .order_by("-created")
            .first()
        )

        if email_template:
            html = email_template.html_body
        else:
            template = "nets_core/email/verification_code.html"

        result = send_email(
            _("Verification code"),
            [instance.user.email],
            template,
            {"button_link": button_link, "user": instance.user},
            html=html,
            to_queued=False,
        )
        logger.info(
            f"Verification code email to {instance.user.email} with result {result}"
        )

    else:
        if instance.verified:
            # send notification to user about new login
            template = "nets_core/email/new_login.html"
            send_email(
                _("New login"),
                [instance.user.email],
                template,
                {"user": instance.user, "ip": str(instance.ip), "device": instance.device},
                to_queued=False,
            )

            message = _("New login to your account from ip address") + " " + instance.ip
            data = {"type": "login", "ip": str(instance.ip), "device": ""}
            if instance.device:
                message += " " + _("using device") + " " + instance.device.__str__()
                data["device"] = f"{instance.device.name}"
                data["device_id"] = f"{instance.device_id}"
            title = f"{_('New login')} {instance.user.username}"
            send_user_device_notification(
                instance.user, title=title, message=message, data=data
            )
